use std::{f32::consts::PI, time::Duration};

use bevy::prelude::{Transform, Vec2};
use bevy_tweening::{
    lens::{TransformPositionLens, TransformRotateZLens},
    EaseFunction, Tween, TweeningType,
};

use crate::turtle::{Angle, TurtleGraphElement, TurtleState};

pub fn turtle_turn(
    state: &mut TurtleState,
    angle_to_turn: f32,
) -> (Option<Tween<Transform>>, Option<TurtleGraphElement>) {
    let start = state.heading;
    let end = state.heading + (angle_to_turn * PI / 180.);
    let animation = Tween::new(
        // Use a quadratic easing on both endpoints
        EaseFunction::QuadraticInOut,
        TweeningType::Once,
        // Animation time
        Duration::from_millis(500),
        // Rotate the turtle
        TransformRotateZLens { start, end },
    )
    .with_completed_event(state.index as u64);
    // Dont move and draw
    let line = TurtleGraphElement::Noop;
    // Update the state
    state.heading = end % (2. * PI);
    (Some(animation), Some(line))
}

pub fn turtle_move(
    state: &mut TurtleState,
    length: f32,
) -> (Option<Tween<Transform>>, Option<TurtleGraphElement>) {
    let start = state.start;
    let end = state.start + (Vec2::from_angle(state.heading) * length);
    let turtle_movement_animation = Tween::new(
        // accelerate and decelerate
        EaseFunction::QuadraticInOut,
        TweeningType::Once,
        // later to be controlled by speed
        Duration::from_millis(500),
        // set the start and end of the animation
        TransformPositionLens {
            start: start.extend(0.),
            end: end.extend(0.),
        },
    )
    .with_completed_event(state.index as u64);
    // The line for animating and drawing
    let line = if state.drawing {
        TurtleGraphElement::TurtleLine { start, end }
    } else {
        TurtleGraphElement::Noop
    };
    state.start = end;
    (Some(turtle_movement_animation), Some(line))
}

pub fn turtle_circle(
    state: &mut TurtleState,
    radius: f32,
    angle: Angle,
) -> (Option<Tween<Transform>>, Option<TurtleGraphElement>) {
    let radius_tuple = Vec2::ONE * radius.abs();
    dbg!(state.start, radius_tuple);
    let center = state.start
        + (Vec2::new(radius, 0.).rotate(Vec2::from_angle((state.heading + 90.).to_radians())));
    dbg!(center);
    let turtle_movement_animation = Tween::new(
        // accelerate and decelerate
        EaseFunction::QuadraticInOut,
        TweeningType::Once,
        // later to be controlled by speed
        Duration::from_millis(500),
        // set the start and end of the animation
        TransformPositionLens {
            start: center.extend(0.),
            end: center.extend(0.),
        },
    )
    .with_completed_event(state.index as u64);
    // The line for animating and drawing
    let line = if state.drawing {
        TurtleGraphElement::TurtleCircle {
            center,
            radii: radius_tuple,
            angle,
            start: state.start,
            end: state.start + 200.,
        }
    } else {
        TurtleGraphElement::Noop
    };
    // TODO update end_position : state.start = end;
    (Some(turtle_movement_animation), Some(line))
}
